/*function Collection(keysDef, log) {
	this.keys = keysDef;
	this.objects = {};
	
	this.getKeyForObject = function(object) {
		if(this.keys.length == 0)
			return null;
			
		var key = object[this.keys[0]];
		for(var idx = 1; idx < this.keys.length; idx++)
			key = key + '|' + object[this.keys[idx]];
			
		return key;
	};
	
	this.add = function(object) {
		if(this.objects === undefined)
			this.objects = {};
		this.objects[this.getKeyForObject(object)] = object;
	};
	
	this.get = function(key) {
		if(this.objects === undefined)
			return undefined;
			
		return this.objects[key];
	};
	
	this.getByIndex = function(index) {
		if(this.objects === undefined)
			return undefined;
		var key = Object.keys(this.objects)[index];
		
		return this.get(key);
	}
	
	this.length = function() {
		if(this.objects === undefined)
			return 0;
			
		return Object.keys(this.objects).length;
	}

	this.remove = function(key) {
		delete this.objects[key];
	}
	
	this.removeAll = function() {
		this.objects = {};
	};
	
	this.toArray = function() {
		var array = $.map(this.objects, function(value, index) {
			return [value];
		});
		
		return array;
	}
	
	this.update = function(object) {
		this.add(object);
	};	
};*/

ConnectionState = {
    UNCONNECTED: 	'UNCONNECTED',
    UNCONNECTED: 	'CONNECTING',
	CONNECTED: 		'CONNECTED',
	ERROR:			'ERROR'
};

QueryState = {
    STARTING: 		'STARTING',
    ACTIVE: 		'ACTIVE',
    UNCONNECTED: 	'UPDATING',
	ERROR:			'ERROR',
	SUCCESS:		'SUCCESS',
    UNSUBSCRIBED: 	'UNSUBSCRIBED'
};

QueryType = {
    SUBSCRIPTION: 	'SUBSCRIPTION',
    SNAPSHOT: 		'SNAPSHOT'
};

function HTTPControllerConnection(config) {
	// Debug flag
	var debug = false;
	
	// Set the configuration parameters
	var scope 		= config.scope;
	var log 		= config.log;
	var controller 	= config.controller;
	var configurl 	= config.configurl;
	
	// Initialize objects
	var queries 	= new Collection(['id']);
	
	// Initialize a global variable as well for controller name
	this.controller = controller;
	
	// Initialize a local variable to hold current reference
	var self 		= this;
		
	/** Load the query configuration and process */
	this.init = function() {
		setTimeout(function() {
			// If there is a configurl, load queries
			if(configurl != null) {
				$.ajax({
					type: 'GET',
					url: configurl,
					dataType: 'json',
					success: function(data) {
						// Add each loaded query to the queries collection
						for(var idx = 0; idx < data.queries.length; idx++) {
							var query = data.queries[idx];
							queries.add(query);
						}
							
						// Process the queries
						self.processConfiguredQueries();
					}
				});	
			}
		},100);
	};
	
	/** Retrieve a list of tables in the system */
	this.listTables = function(key, includeSystemTables) {
	};
	
	/** Process all configured queries and run them as snapshots */
	this.processConfiguredQueries = function() {
		for(var idx = 0; idx < queries.length(); idx++) {
			var query = queries.getByIndex(idx);
			this.snapshot(query);
		}
	};
	
	/** Adds the query unless keep is false. Then it executes query as a snapshot */
	this.snapshot = function(query, keep) {
		// Broadcast a snapshot change
		scope.$broadcast('query:' + query.id + ':snapshot', 'change');
		
		if(keep != false) queries.add(query);
		query.status = QueryState.STARTING;
		query.type = QueryType.SNAPSHOT;
		query.objects = new Collection(['tid']);
		scope.$broadcast('query:' + query.id + ':status', query);
		
		var request = query.request == undefined ? {} : query.request;
		var url = '/webapp/relay/' + query.url;
		$.ajax({
			type: "POST",
			url: url,
			data: JSON.stringify(request),
			dataType: "text json",
			success: function(xhr, textStatus, errorThrown) {
				query.schema = xhr.response.schema;
				query.status = QueryState.SUCCESS;
				scope.$broadcast('query:' + query.id + ':status', query);

				if(Object.keys(xhr.response.data).length > 0) {
					// Loop over the event data and create objects
					//   Broadcast an event for each object
					for(var keyIdx = 0; keyIdx < Object.keys(xhr.response.data).length; keyIdx++) {
						var key = Object.keys(xhr.response.data)[keyIdx];
						if(xhr.response.data[key].length == undefined) {
							var object = xhr.response.data[keyIdx];
							query.objects.add(object);
							scope.$broadcast('query:' + query.id + ':snapshot', 'onSuccess:object', event, object);
						}
						else {
							for(var dataIdx = 0; dataIdx < xhr.response.data[key].length; dataIdx++) {
								var object = xhr.response.data[key][dataIdx];
								query.objects.add(object);
								scope.$broadcast('query:' + query.id + ':snapshot', 'onSuccess:object', event, object);
							}
						}
					}					
				}
				
				scope.$broadcast('query:' + query.id + ':snapshot', 'onSuccess', event);

				if(debug == true)
					log.debug('[' + controller + '] ' + 'query:' + query.id + ':snapshot:onSuccess');					
			},
			error: function(xhr, textStatus, errorThrown) {
				log.error(xhr);
				query.error = error.message;
				query.status = QueryState.ERROR;
				scope.$broadcast('query:' + query.id + ':status', query);
				scope.$broadcast('query:' + query.id + ':snapshot', 'onError', error);
			}
		});		
	};
	
	/** Removes an existing query from the list of registered queries. If the query 
	 *  is a subscription it will be unsubscribed first.
	 */
	this.remove = function(query) {
		var id = query.id;
		queries.remove(query.id);	
		scope.$broadcast('query:' + id + ':remove', 'success');
	};
	
	/** Closes this connection */
	this.close = function() {
		// Remove all queries
		for(var idx = queries.length() - 1; idx >= 0; idx--) {
			var query = queries.getByIndex(idx);
			this.remove(query);
		}
	};	

	/** Retrieve the query by id */
	this.getQuery = function(queryId) {
		return queries.get(queryId);
	};	
}