app.directive('pieChart', function($log, $window, PieChart, Overlay, EventListener) {
	return {
		templateUrl: './templates/live-chart.html',
		restrict: 'E',
		replace: false,
		transclude: true,
		scope: {
			'query':	'=',
			'modal':	'=',
			'selected':	'='
		},
		controller: function($scope) {
		},
		link: function(scope, element, attrs, ctrl, transclude) {
			// Change the scope of the transcluded tooltip so that it is read from
			//   the directive scope instead of the parent scope
			transclude(scope, function(clone, scope) {
				element.append(clone);
			});
			
			// Set the query ID to the scope
			scope.queryId = attrs.queryId;
			
			// Set the element ID to scope if it exists
			if(attrs.id != undefined) scope.elementId = attrs.id;
			
			// Read the chart options from attributes
			scope.options = {
				"uid":				"tid",
									// Mandatory. Object property that is unique and passed into the chart for comparison.
									
				"filter":			(attrs.filter != undefined && attrs.filter.length > 0),
									// Optional. Allow data filtering TODO.
				
				"modal":			(attrs.modal != undefined && attrs.modal.length > 0),
									// Optional. Display query modal.
				
				"selectable":		(attrs.selected != undefined && attrs.selected.length > 0),
									// Optional. User can select chart element and object will be assigned
									//   to this scope variable
								
				"key":				attrs.key,
									// Optional. Object property for chart element key.
				
				"value":			attrs.value,
									// Mandatory. Object property for chart element value.
				
				"animation":		attrs.animation == undefined || attrs.animation == "" ? 
										0 : 
										parseInt(attrs.animation),
									// Optional. Animation duration for build of a slice on draw or update.
																		
				"chartTitle":		attrs.chartTitle,
									// Optional. Title to apply to the chart.
									
				"chartTitleOffset":	attrs.chartTitleOffset == undefined ?
										undefined :
										eval("(" + attrs.chartTitleOffset + ")"),
									// Optional. Offset to apply to the default title position. {x:100, y:100}
									
				"color":			attrs.color == undefined ?
										d3.scale.category10() :
										eval(attrs.color),
									// Optional. Color scale to apply. 
									//   Ordinal: d3.scale.category10(), d3.scale.category20(), d3.scale.category20b(), d3.scale.category20c()
									//   Ordinal: d3.scale.category10().domain(['B','N','Q','X','Y']).range(['blue','orange','purple','brown','yellow'])
									//   Ramp: d3.scale.linear().domain([0,100]).range(['blue','red'])
									
				"colorValue":		attrs.colorValue == undefined ?
										attrs.key :
										attrs.colorValue,
									// Optional. Object property applied against the color scale.
										
				"donut":			attrs.donut == undefined ? 
										false : 
										eval(attrs.donut),
									// Optional. Display chart as a donut chart.
								
				"donutRatio":		attrs.donutRatio == undefined && attrs.donut == "true" ? 
										0.5 : 
										attrs.donutRatio,
									// Optional. Size of donut hole as ratio of total radius
								
				"growOnHover":		attrs.growOnHover == undefined ? 
										false : 
										eval(attrs.growOnHover),
									// Optional. Grow chart element on hover.
								
				"groupBy":			attrs.groupBy == undefined || attrs.groupBy == "" ?
										false :
										eval(attrs.groupBy),
									// Optional. Object property to group data. May be true or false.
									
				"groupByValue":		(attrs.groupByValue == undefined || attrs.groupByValue == "") && attrs.groupBy == "true" ?
										attrs.label :
										attrs.groupByValue,
									// Optional. Object property to group data.
									
				"groupByColor":		(attrs.groupByColor == undefined || attrs.groupByColor == "") && attrs.groupBy == "true" ?
										d3.scale.category10() :
										eval(attrs.color),
									// Optional. Color scale to apply. See above for possible values.
									
				"labels":			attrs.labels == undefined ? 
										false : 
										eval(attrs.labels),
									// Optional. Display chart element labels.
								
				"labelThreshold":	attrs.labelThreshold == undefined || attrs.labelThreshold == "" ? 
										 0 : 
										parseFloat(attrs.labelThreshold),
									// Optional. Minimum chart element size before label is displayed.
								
				"labelType":		attrs.labelType,
									// Optional. Label type to display. May be "key", "value", or "percent"
								
				"labelFormat":		attrs.labelFormat,
									// Optional. Format to apply to label. d3.format(labelFormat)
									//   Reference: https://github.com/mbostock/d3/wiki/Formatting
								
				"labelPosition":	(attrs.labelPosition == undefined || attrs.labelPosition == "") && attrs.labels == "true" ? 
										"inside" : 
										attrs.labelPosition,
									// Optional. Label positioning. May be "inside", "outside"
								
				"labelRadius":		(attrs.labelRadius == undefined || attrs.labelRadius == "") && attrs.labels == "true" && attrs.labelPosition == "outside" ? 
										25 : 
										attrs.labelRadius,
									// Optional. Label radius delta for outside labels. Ignored for inside labels. This number is added to the computed pie outside radius.
								
				"legend":			attrs.legend == undefined ? 
										false : 
										eval(attrs.legend),
									// Optional. Display legend.

				"legendPosition":	attrs.legendPosition == undefined && attrs.legend == "true" ? 
										"left" : 
										attrs.legendPosition,
									// Optional. Position to display legend. May be "left", "top", "right", "bottom".

				"legendSize":		attrs.legendSize == undefined && attrs.legend == "true" ? 
										18 : 
										parseInt(attrs.legendSize),
									// Optional. Size of the legend rectangle.

				"legendSpacing":	attrs.legendSpacing == undefined && attrs.legend == "true" ? 
										4 : 
										parseInt(attrs.legendSpacing),
									// Optional. Spacing of legend elements.

				"margin":			attrs.margin == undefined || attrs.margin == ""? 
										{top:0, right:0, bottom:0, left:0} : 
										eval("(" + attrs.margin + ")"),
									// Optional. Margin to apply. {top:100, right:20, bottom:10, left:25}

				"quadrant":			attrs.quadrant,
									// Optional. Quadrant chart. May be "N", "S", "E", "W" for 1/2 quadrant. 
									//   Or "NE", "NW", "SE", "SW" for 1/4 quadrant.

				"sorted":			attrs.sorted == undefined ?
										false :
										eval(attrs.sorted),
									// Optional. Sort the data.

				"sortValue":		attrs.sortValue == undefined && attrs.sorted == "true" ?
										attrs.key :
										attrs.sortValue,
									// Optional. Object property to sort on.
										
				"sortDirection":	attrs.sortDirection == undefined && attrs.sorted == "true" ?
										"asc" :
										attrs.sortDirection,
									// Optional. Direction to sort. May be asc or desc.
										
				"tooltips":			attrs.tooltips == undefined ? 
										false : 
										eval(attrs.tooltips)
									// Optional. Display tooltips.
			};
			
			// Declare a chart variable
			var chart = null;

			// Declare a dataset object
			scope.dataset = {};
			
			// Setup an overlay watching the query status
			Overlay.create(scope, scope.query, element);
			
			// Listen to query status messages for schema definition and display the table structure
			scope.$on('query:' + scope.queryId + ':status', function(event, query) {
				if(query.schema == undefined) return;

				// If the chart is already defined, just return
				if(chart != null) return;

				// Create the chart
				chart = PieChart.createChart(scope, element);

				// Draw the chart
				chart.draw();
			});

			// Listen to snapshot events
			scope.$on('query:' + scope.queryId + ':snapshot', function(event, action, serverEvent, object) {
				if(action == 'change') {
					scope.dataset = {};
					chart.draw();
				}
				else if(action == 'onSuccess:object') {
					if(chart == null) return ;
					scope.dataset[object.tid] = object;
					chart.update();
				}
			});
				
			// Listen to subscribe events
			scope.$on('query:' + scope.queryId + ':subscribe', function(event, action, serverEvent, object) {
				if(action == 'onSnapshotStart')
					scope.snapshot = true;
				else if(action == 'onInsert') {						
					if(chart == null) return ;
					scope.dataset[object.tid] = object;
					chart.update();
				}
				else if(action == 'onUpdate') {
					if(chart == null) return ;
					scope.dataset[object.tid] = object;
					chart.update();
				}
				else if(action == 'onDelete') {
					if(chart == null) return ;
					delete scope.dataset[object.tid];
					chart.update();
				}
				if(action == 'onSnapshotEnd')
					scope.snapshot = undefined;
			});
				
			// Listen to unsubscribe events
			scope.$on('query:' + scope.queryId + ':unsubscribe', function(event, message, serverEvent) {
				scope.dataset = {};
				chart.draw();
			});
								
			// Watch selected to update the chart
			scope.$watch('selected', function() {
				if(chart == null) return;
				chart.update();
			});
			
			// Function to calculate and format percentage for display in tooltips
			scope.formatPercentage = function(pct, format) {
				if(format == undefined)
					return pct;
				return d3.format(format)(pct);
			};
			
			// Bind listener on window resize to adjust table heights for auto-sized tables
			EventListener.windowResize(function() {
				if(chart == null) return;
				chart.draw();
			});
			
			// Bind listener on component display for tabs, applications, and visibility-toggle divs
			EventListener.componentDisplay(element, function() {
				if(chart == null) return;
				chart.draw();
			});						
		}
	};	
});
