app.filter('objectFilter', function($log){
	return function(items, search, searchCols, filter, columns){
		if(items == undefined || items.length == 0) return items;
		
		// Setup search column keys
		var searchKeys = searchCols;
		if(searchKeys.length == 0) searchKeys = Object.keys(items[0].data);
		
		var filtered = [];
		angular.forEach(items, function(item) {
			var include = true;

			// Search text, search any, inexact match
			if(search != undefined && search.length > 0 && searchKeys.length > 0) {
				var searchInclude = false;
				for(var idx = 0; idx < searchKeys.length; idx++) {
					if(item.data[searchKeys[idx]] != undefined && (item.data[searchKeys[idx]] + "").toLowerCase().indexOf(search.toLowerCase()) >= 0) {
						searchInclude = true;
						break;
					}
				}
				include = searchInclude;
			}
			
			// Filter object, exact match
			if(include == true && filter != undefined && Object.keys(filter).length > 0) {			
				var filterInclude = true;
				var filterKeys = Object.keys(filter);
				for(var idx = 0; idx < filterKeys.length; idx++) {
					if(item.data[filterKeys[idx]] != filter[filterKeys[idx]]) {
						filterInclude = false;
						break;
					}
				}
				include = filterInclude;
			}
			
			// Column search text, inexact match and inequality
			if(include == true) {
				var fieldSearchInclude = true;
				for(var idx = 0; idx < columns.length; idx++) {
					var column = columns[idx];
					if(column.search == undefined) continue;
					var d = item.data[column.field.name];
					var t = column.search;
					if(t.indexOf('>') == 0 || t.indexOf('<') == 0 || t.indexOf('!') == 0 || t.indexOf('==') == 0) {
						var expression = t;//.replace(/([.*+?^=!:${}()|\[\]\/\\])/g, "\\$1");
						var expression = expression
							.replace(new RegExp('>', 'g'), ' d >')
							.replace(new RegExp('<', 'g'), ' d <')
							.replace(new RegExp('!', 'g'), ' d !')
							.replace(new RegExp('==', 'g'), ' d ==');							
						//$log.debug(expression);
						if(d != undefined && eval(expression) == false) {
							fieldSearchInclude = false;
							break;
						}
					}
					else if((d + "").toLowerCase().indexOf(t) == -1) {
						fieldSearchInclude = false;
						break;
					}
				}			
				include = fieldSearchInclude;
			}
			
			if(include == true) filtered.push(item);
		});
		return filtered;
	};
});

app.filter('objectLimitTo', function($log){
	return function(items, pagination){
		if(items == undefined || pagination == undefined) return items;
		var begin = pagination.begin == undefined ? 0 : pagination.begin;
		var end = pagination.end == undefined || pagination.end == 0 ? begin + pagination.size : pagination.end;
		return items.slice(begin, end);
	};
});

app.filter('objectOrderBy', function($log) {
	return function(items, order, snapshot) {
		if(snapshot == true) return items;
		var filtered = [];
		angular.forEach(items, function(item) {
			filtered.push(item);
		});

		var compare = function(a, b, dir, field) {
			return dir * (a.data[field] == b.data[field] ? 0 : a.data[field] < b.data[field] ? 1 : -1);
		};
			
		// This is default sort algorithm
		/*filtered.sort(function (a, b) {
			var compare = function(a, b, dir, field) {
				return dir * (a[field] == b[field] ? 0 : a[field] > b[field] ? 1 : -1);
			}
			
			for(var idx = 0; idx < order.length; idx++) {
				var col = order[idx];
				var dir = col.ordering == 'desc' ? -1 : 1;
				var v = compare(a, b, dir, col.field.name);
				if(v != 0) return v;
			}
			return 0;
		});*/
		// return filtered;
		// End default sort
		
		// This is merge sort algorithm, might be more efficient
		var algo = function(a, b) {
			for(var idx = 0; idx < order.length; idx++) {
				var col = order[idx];
				var dir = col.ordering == 'desc' ? -1 : 1;
				var v = compare(a, b, dir, col.field.name);
				if(v != 0) return v;
			}
			return 0;
		};
		
		var sort = function(array) {
			var len = array.length;
			if(len < 2) 
				return array;

			var pivot = Math.ceil(len/2);
			return merge(sort(array.slice(0,pivot)), sort(array.slice(pivot)));
		};

		var merge = function(left, right) {
			var result = [];
			while((left.length > 0) && (right.length > 0)) {		
				if(algo(left[0], right[0]) > 0)
					result.push(left.shift());
				else
					result.push(right.shift());
			}

			result = result.concat(left, right);
			return result;
		};

		return sort(filtered);
		// End merge sort
	};
});

app.filter('orderByInt', function($log) {
	return function(items, property, direction) {
		var filtered = [];
		angular.forEach(items, function(item) {
			filtered.push(item);
		});

		var compare = function(a, b, dir, field) {
			return dir * (parseInt(a[field]) == parseInt(b[field]) ? 0 : parseInt(a[field]) < parseInt(b[field]) ? 1 : -1);
		};
			
		filtered.sort(function (a, b) {
			var dir = direction == 'desc' ? 1 : -1;
			return compare(a, b, dir, property);
		});
		return filtered;
	};
});

app.filter('orderByTimestamp', function($log, SimpleDateFormat) {
	return function(items, property, direction) {
		var filtered = [];
		angular.forEach(items, function(item) {
			filtered.push(item);
		});

		var compare = function(a, b, dir, field) {
			//return dir * (parseInt(SimpleDateFormat.toTimestamp(a[field])) == parseInt(SimpleDateFormat.toTimestamp(b[field])) ? 0 : 
			//	parseInt(SimpleDateFormat.toTimestamp(a[field])) < parseInt(SimpleDateFormat.toTimestamp(b[field])) ? 1 : -1);
			return dir * a[field] == b[field] ? 0 : a[field] > b[field] ? 1 : -1;
		};
			
		filtered.sort(function (a, b) {
			var dir = direction == 'desc' ? 1 : -1;
			return compare(a, b, dir, property);
		});
		return filtered;
	};
});

app.filter('orderByProperty', function($log, SimpleDateFormat) {
	return function(items, property, direction) {
		var filtered = [];
		angular.forEach(items, function(item) {
			filtered.push(item);
		});

		var compare = function(a, b, dir, field) {
			return dir * (a[field] == b[field] ? 0 : a[field] < b[field] ? 1 : -1);
		};
			
		filtered.sort(function (a, b) {
			var dir = direction == 'desc' ? 1 : -1;
			return compare(a, b, dir, property);
		});
		return filtered;
	};
});

app.filter('filterByProperty', function($log) {
	return function(items, property, value) {
		var filtered = [];
		angular.forEach(items, function(item) {
			if(item[property] == value)
				filtered.push(item);
		});
		return filtered;
	};
});

