app.service('Properties', function ($rootScope, $log) {
	var properties = {};

	return {
		getProperty: function (key) {
			return properties[key];
		},
		setProperty: function(key,value) {
			properties[key] = value;
			$rootScope.$broadcast(key, value);
		},
		getSessionProperty: function (key) {
			return window.sessionStorage[key];
		},
		setSessionProperty: function(key,value) {
			window.sessionStorage[key] = value;
		}		
	};
});

app.service('Application', function($log) {
	return {
		show: function(nav) {
			$('.application').hide();
			$('#' + nav + '-app').show();
		}		
	};
});

app.service('SimpleDateFormat', function($log) {
	return {
		format: function(data, format) {			
			if(data == undefined) return "";				
			if(format == undefined) return data;
			
			var date = data;
			if(!(date instanceof Date)) {
				date = new Date(date);
				// Firefox hack
				//date = new Date(data.replace(' ', 'T'));
			}
				
			// Array of short month names
			var shortMonth = ['Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec'];	
			
			// Array of date format tokens
			var o = {
				"yyyy+" : 	date.getFullYear(),
				"yy+" : 	(date.getFullYear() + "").substring((date.getFullYear() + "").length - 2),
				"dd+" : 	(date.getDate() < 10 ? "0" : "") + date.getDate(),
				"HH+" : 	(date.getHours() < 10 ? "0" : "") + date.getHours(),
				"mm+" : 	(date.getMinutes() < 10 ? "0" : "") + date.getMinutes(),
				"ss+" : 	(date.getSeconds() < 10 ? "0" : "") + date.getSeconds(),
				"S" : 		(date.getMilliseconds() < 10 ? "00" : date.getMilliseconds() < 100 ? "0" : "") + date.getMilliseconds(),
				// Month has to come last and has to be in this order due to the string substitution
				"MMM+" : 	shortMonth[date.getMonth()],
				"MM+":		(date.getMonth() + 1 < 10 ? "0" : "") + (date.getMonth() + 1)
			}

			var formatted = format;
			for(var k in o) {
				if(new RegExp("("+ k +")").test(formatted))
					formatted = formatted.replace(RegExp.$1,o[k]);
			}
				
			return formatted;
		}
	};
});

app.service('NumberFormat', function() {
	return {
		format: function(data, format) {			
			if(format === undefined || data === null || data === undefined)
				return data;

			var f = format;
			if(f.indexOf('.') == -1)
				f = f + '.';
			var parts = f.split('.');			
			if(parts.length == 2)
				return parseFloat(data).toFixed(parts[1].length);
			
			return data;
		}
	};
});

app.service('SeparateDigits', function() {
	return {
		format: function(data,start,length) {			
			if(data === null || data === undefined)
				return data;

			digits = data.toString().substr(start,length);
			
			return digits;
		}
	};
});

app.service('AlertMe', function() {
	
});




app.service('Overlay', function($log) {
	return {
		create: function(scope, queryId, element) {

			var listener = scope.$on('query:' + queryId + ':status', function(event, query) {
				var parentContainer = $(element).closest('.overlay-container');
				if(parentContainer[0].ids == undefined) parentContainer[0].ids = [];
				if(parentContainer[0].ids.indexOf(queryId) == -1 && (query.status == undefined || query.status == QueryState.STARTING)) {
					parentContainer[0].ids.push(queryId);
					if(parentContainer[0].ids.length == 1) {
						var over = "<div class='div-preloader'></div>";
						$(over).appendTo(parentContainer.css("position", "relative"));	
					}
				}
				else if(query.status == QueryState.ACTIVE || query.status == QueryState.SUCCESS) {
					if(parentContainer[0].ids.length == 1) {
						parentContainer[0].ids = undefined;
						$(parentContainer).find('.div-preloader').remove();
						listener(); // remove the listener
					}
					else {
						parentContainer[0].ids.splice(parentContainer[0].ids.indexOf(queryId),1);
					}
				}
			});
		}		
	};
});

app.service('EventListener', function($log, $window, $timeout) {
	return {
		windowResize: function(resize) {
			// Bind listener on window resize to adjust table heights for auto-sized tables
			angular.element($window).bind('resize', function(){
				var waitForFinalEvent = (function () {
					var timers = {};
					return function (callback, ms, uniqueId) {
						if (!uniqueId)
							uniqueId = "Don't call this twice without a uniqueId";
						if (timers[uniqueId])
							clearTimeout (timers[uniqueId]);
						timers[uniqueId] = setTimeout(callback, ms);
					};
				})();

				waitForFinalEvent(function(){
					if(resize != undefined) resize();
				}, 10, new Date().getTime() + "");
			});	
		},

		componentDisplay: function(element, display) {
			// This table may be in a tab. If it is, bind listener on document ready to redraw the table
			angular.element($(element).closest('.tab-pane')).ready(function() {
				// Delay setting the callback until document renders
				$timeout(function(){
					var parentId = $(element).closest('.tab-pane').attr('id');
					if(parentId != undefined && parentId != "{{thisQuery.id}}") {
						$('a[href=#' + parentId + ']').on('shown.bs.tab', function(e) {
							if(display != undefined) display();
						});
					}
				},100);
			});	
			
			// This table may be in a hideable application. If it is, bind listener on show of the application 
			//   to redraw the table
			angular.element($(element).closest('.application')).ready(function() {
				// Delay setting the callback until document renders
				$timeout(function(){
					// Listen for show events. This only works because of addition of custom jQuery handler for
					//   show/hide (see top of ng-directives.js.
					var application = $(element).closest('.application').on('show', function() {
						if(display != undefined)
							display();
					});
				},100);
			});	
			
			// This table may be in a hideable div. If it is, bind listener on show of the div 
			//   to redraw the table
			//angular.element($(element).closest('.visibility-toggle')).ready(function() {
				// Delay setting the callback until document renders
				$timeout(function(){
					// Listen for show events. This only works because of addition of custom jQuery handler for
					//   show/hide (see top of ng-directives.js.
					var application = $(element).closest('.visibility-toggle').on('show', function() {
						if(display != undefined) display();
					});
					$(element).closest('.visibility-toggle').trigger("show");					
				},100);
			//});			
		}
	};
});

app.service('Map', function($log, $compile) {
	return {
		configurePopup: function(scope, element, component) {
			// Create the popup if specified
			if($(element).find('livemap-popup').length > 0) {
				// Create an isolate scope
				var contentScope = scope.$new(true);
				contentScope.object = component.data;
				contentScope.component = component;
				
				// Compile the popup template against the isolate scope and apply
				var content = $compile(scope.options.popup)(contentScope);	
				contentScope.$applyAsync();
				contentScope.$destroy();
					
				// Bind the popup to the node
				component.node.bindPopup(content.html());
			}		
		},

		updatePosition: function(scope, component) {
			var object = component.data;
			var position = eval(scope.options.position);
			if(position[0] != null && position[1] != null) {
				var latLng = L.latLng(position);
				component.node.setLatLng(latLng);
				if(component.node.getPopup() != undefined)
					component.node.getPopup().setLatLng(latLng);
			}		
		},
		
		updateTitle: function(scope, ctrl, component) {
			var object = component.data;
			component.title = eval(ctrl.getObjectTitleExpression());
		},
		
		getColor: function(options, property, object, component) {
			if(options[property] == undefined) return undefined;

			// Determine what kind of color definitions we have
			if(options[property].indexOf('{') == 0 && options[property].indexOf('}')  > -1) {
				var colorObj = eval('(' + options[property] + ')');
				var colors = Object.keys(colorObj);
				for(var idx = 0; idx < colors.length; idx++) {
					var color = colors[idx];
					var value = colorObj[color];
					if(value == true) return color;
				}
				return '#03f';
			}
			else
				return options[property];
		},

		getWeight: function(options, property, object, component) {
			if(options[property] == undefined) return undefined;

			// Determine what kind of weight definitions we have
			if(options[property].toString().indexOf('{') == 0 && options[property].toString().indexOf('}')  > -1) {
				var weightObj = eval('(' + options[property] + ')');
				var weights = Object.keys(weightObj);
				for(var idx = 0; idx < weights.length; idx++) {
					var weight = weights[idx];
					var value = weightObj[weight];
					if(value == true) return weight;
				}
				return 3;
			}
			else
				return options[property];
		},

		getPathOptions: function(attrs) {
			var options = {
				"className":		attrs.classes == undefined || attrs.classes.length == 0 ?
										undefined :
										attrs.classes,
									// Optional. Class name to apply to the element.
								
				"clickable":		attrs.clickable == undefined || attrs.clickable.length == 0 ?
										true :
										eval(attrs.clickable),
									// Optional. Make the object clickable. 
								
				"stroke":			attrs.stroke == undefined || attrs.stroke.length == 0 ?
										true :
										eval(attrs.stroke),
									// Optional. Whether to draw the stroke along the path. Set to false to disable borders on polygons or circles.
								
				"color":			attrs.color == undefined || attrs.color.length == 0 ?
										'#03f' :
										attrs.color,
									// Optional. Stroke color. May be a constant or a conditional object:
									//   For example: {'red': object.status == 'Approach', 'yellow': object.status == 'Circling', 'green': true}: 
									//   Will give the color for the first expression that evaluates to true. Expression must evaluate to
									//   boolean and can use "object" to refer to the object being rendered.

								
				"dashArray":		attrs.dashArray == undefined || attrs.dashArray.length == 0 ?
										null :
										attrs.dashArray,
									// Optional. String that defines a stroke dash pattern.
									// Reference: https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/stroke-dasharray
								
				"lineCap":			attrs.lineCap == undefined || attrs.lineCap.length == 0 ?
										null :
										attrs.lineCap,
									// Optional. String that defines shape to use at the end of the stroke.
									// Reference: https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/stroke-linecap
								
				"lineJoin":			attrs.lineJoin == undefined || attrs.lineJoin.length == 0 ?
										null :
										attrs.lineJoin,
									// Optional. String that defines shape to use at corners of a stroke.
									// Reference: https://developer.mozilla.org/en-US/docs/Web/SVG/Attribute/stroke-linejoin
								
				"opacity":			attrs.opacity == undefined || attrs.opacity.length == 0 ?
										0.5 :
										eval(attrs.opacity),
									// Optional. Opacity for the line stroke.

				"weight":			attrs.weight == undefined || attrs.weight.length == 0 ?
										5 :
										attrs.weight,
									// Optional. Stroke width in pixels.

				"fill":				attrs.fill == undefined || attrs.fill.length == 0 ?
										false :
										eval(attrs.fill),
									// Optional. Whether to fill the path. Set to false to disable fill on polygons or circles.
								
				"fillColor":		attrs.fillColor == undefined || attrs.fillColor.length == 0 ?
										'#03f' :
										attrs.fillColor,
									// Optional. Fill color. May be a constant or a conditional object:
									//   For example: {'red': object.status == 'Approach', 'yellow': object.status == 'Circling', 'green': true}: 
									//   Will give the color for the first expression that evaluates to true. Expression must evaluate to
									//   boolean and can use "object" to refer to the object being rendered.
								
				"fillOpacity":		attrs.fillOpacity == undefined || attrs.fillOpacity.length == 0 ?
										0.2 :
										eval(attrs.fillOpacity),
									// Optional. Opacity for the fill.

				"zIndexOffset":		attrs.zIndexOffset == undefined || attrs.zIndexOffset.length == 0 ?
										undefined :
										attrs.zIndexOffset,
									// Optional. z-index offset within the SVG group.
			};
			return options;
		},
		
		updateVertexGroup: function(groupTid, object, options, vertexGroupSet, vertexSet) {
			// Try to retrieve the existing vertexGroup
			var vertexGroup = vertexGroupSet[groupTid];
		
			// If it's not found, then create it
			if(vertexGroup == undefined) {
				isnew = true;
				vertexGroup = {
					tid:		groupTid, 	// Set tid which will be unique for the vertexGroupSet. 
											//   This will also get propagated up to the layer.
					vertices:	[]			// Define an array of vertices
				};
				
				// Iterate over all the group by properties and copy them all from the object
				//   to the vertexGroup
				for(var idx = 0; idx < options.groupBy.length; idx++)
					vertexGroup[options.groupBy[idx]] = object[options.groupBy[idx]];	

				// Iterate over all the group properties and copy them all from the object to
				//   the vertexGroup
				for(var idx = 0; idx < options.groupProperties.length; idx++)
					vertexGroup[options.groupProperties[idx]] = object[options.groupProperties[idx]];
			}
			
		
			// Create a vertex for the object
			var vertex = {
				data:	object,		// Set the data
				parent:	vertexGroup.tid // Set the parent vertexGroup for reference later
			}
			
			// Add the vertex to the vertexGroup
			vertexGroup.vertices.push(vertex);

			// Put the vertex in the vertex set for access later
			vertexSet[object.tid] = vertex;
			
			return vertexGroup;
		},
	
		getVertexKey: function(object, options) {
			// Setup the default key
			var key = "1";

			// If grouping is enabled, update the key
			if(options.groupBy.length > 0) {
				key = object[options.groupBy[0]];
				for(var idx = 1; idx < options.groupBy.length; idx++)
					key = key + "|" + object[options.groupBy[idx]];
			}			
			return key;
		},
	};
});

app.service('Color', function($log) {
	return {
		hsvToRgb: function(h, s, v) {
			var r, g, b;
			var i;
			var f, p, q, t;

			// Make sure our arguments stay in-range
			h = Math.max(0, Math.min(360, h));
			s = Math.max(0, Math.min(100, s));
			v = Math.max(0, Math.min(100, v));

			// We accept saturation and value arguments from 0 to 100 because that's
			// how Photoshop represents those values. Internally, however, the
			// saturation and value are calculated from a range of 0 to 1. We make
			// That conversion here.
			s /= 100;
			v /= 100;

			if(s == 0) {
				// Achromatic (grey)
				r = g = b = v;
				return [Math.round(r * 255), Math.round(g * 255), Math.round(b * 255)];
			}

			h /= 60; // sector 0 to 5
			i = Math.floor(h);
			f = h - i; // factorial part of h
			p = v * (1 - s);
			q = v * (1 - s * f);
			t = v * (1 - s * (1 - f));

			switch(i) {
				case 0:
					r = v;
					g = t;
					b = p;
					break;

				case 1:
					r = q;
					g = v;
					b = p;
					break;

				case 2:
					r = p;
					g = v;
					b = t;
					break;

				case 3:
					r = p;
					g = q;
					b = v;
					break;

				case 4:
					r = t;
					g = p;
					b = v;
					break;

				default: // case 5:
					r = v;
					g = p;
					b = q;
			}	 
			return [Math.round(r * 255).toString(16), Math.round(g * 255).toString(16), Math.round(b * 255).toString(16)];
		},	
	};
});



